#include <Adafruit_NeoPixel.h>
#include <math.h>

// Display a waving flag on a 48x16 LED array consisting of three 16x16 WS2812b arrays.
// This version uses separate data pins for each panel, so that less memory is needed by the Neopixel library, and Arduino Uno can be used.

#define PIN      6  // change to whatever pin you are using to send data to the LED array.  This is now replaced by the array panelPin[]
#define N_LEDS 256  // number of LEDs in one panel.
#define imageRows 16  // the size of the image that is going to be displayed.  Height normally 16.
// number of rows and columns in each LED panel
#define numRows 16
#define numCols 16
// number of rows and columns in the flag image
#define flagRows 48
#define flagCols 96
#define brightness 70  // 255 would be maximum brightness.  I intend to use this at night, so I set the brightness much lower than the maximum.
#define ncycle 3  // this is the number of wave cycles across the display

Adafruit_NeoPixel strip = Adafruit_NeoPixel(N_LEDS, PIN, NEO_GRB + NEO_KHZ800);

int numPanels = 3; 
int panelPin[3] = {5,6,7};  // Panel 0 connected to pin 5, panel 1 to pin 6, panel 2 to pin 7.
int displayCols = numCols*numPanels;
int pixel = 0;
int imageRow = 0;
int imageCol = 0;
int r=0;
int g=0;
int b=0;
int shade=128;
int x=0;
int y=0;
int xd = 0;
int xdOff = displayCols/ncycle/4;  // offset between displacement and shading
int xd2 = 0;
int px = 0;
bool yEven = true;
int panel = 0;
int value = 0;
uint32_t c = 0;  // used for storing color values
long randValue;  // for later use, not used in this sketch
float ymag = 2.0;  // magnitude of wave in vertical direction (pixels)
float slope = 5.0;  // slope of wave - sinewave will be offset horizontally by 1 pixel every 5th row.
float fcycle = float(ncycle);  // this is the number of wave cycles across the display 
int yoff = int(ymag);  // integer value for later calculations
int shadeScale = 6/yoff;  // controls the depth of the shading effect.

byte ysine[16][48];  // sinewave values

const uint8_t flag[flagRows][flagCols][3] PROGMEM = 
#include "Canada_flag_96x48.h"   // Canada flag image.  The image is twice the resolution of the LED display, plus 8 pixels top and bottom to allow margin for waving.
;

void setup() {
  strip.begin();
  for (panel = 0; panel < numPanels; panel++) {
    strip.setPin(panelPin[panel]);
    strip.show();         // Turn OFF all pixels ASAP
  }
  strip.setBrightness(brightness); // Set BRIGHTNESS

// precompute the sinewave pattern of pixel offsets
  for (x = 0; x<displayCols; x++) {
    for (y=0; y<numRows; y++) {
      ysine[y][x] = byte((sin((float(x)-float(y)/slope)/24.0*fcycle*3.141592654)+1.0)*ymag*8.0+0.5);
    }
  }
}

void loop() {
  
  // waving flag animation

  // update the display, one pixel at a time
  
   for(panel=0; panel<numPanels; panel++) {
    yEven = true;
    px = panel*numCols;
    for (y=0; y<numRows; y++) {
      for (x=0; x<numCols; x++) {
        xd = x + px;  // x coordinate of the LED in the full 48x16 array
        xd2 = xd + xdOff;  // the shading is shifted from the displacement by 1/4 of a cycle.  Maximum shading occurs at maximum slope. 
        if (xd2 >= displayCols) {
          xd2 -= displayCols;
        }
        value = ((int(ysine[y][xd]))>>3) - yoff;
        shade = 128-int(ysine[y][xd2])*shadeScale;  // Shading is used to make the waving look more realistic.  shade=128 is full brightness.
        imageRow = flagRows - y*2 + value - 8;  // subtract 8 because the flag image is 16 pixels larger than twice the display size
        imageCol = xd*2 + (value>>1);
        if ((imageCol < 0)) {  // Draw red (past the left edge of the flag image).  Change to black or a different color if your flag is not red on the left edge.
//          r = 0;
//          g = 0;
//          b = 100;
          r = (213*shade)>>7;  // multiply by shade value (which is maximum 128) and divide by 128.
          g = (43*shade)>>7;
          b = (30*shade)>>7;
        }
        else if (imageCol >= flagCols) {  // draw red (past the right edge of the flag image.  Change to black or a different color if your flag is not red on the right edge.
          r = (213*shade)>>7;
          g = (43*shade)>>7;
          b = (30*shade)>>7;
        }
        else {  // draw flag
          r = (int(pgm_read_byte_near(&flag[imageRow][imageCol][0]))*shade)>>7;
          g = (int(pgm_read_byte_near(&flag[imageRow][imageCol][1]))*shade)>>7;
          b = (int(pgm_read_byte_near(&flag[imageRow][imageCol][2]))*shade)>>7;
        }
        if (yEven) {  // y is even.  Pixel numbers increment in forward order.  Change this section if your panel is oriented differently from mine.
          pixel = y*numCols + x;
        }
        else {  // y is odd.  Pixel numbers decrease with x (reverse part of serpentine path)
          pixel = (y+1)*numCols - x - 1;
        }
        c = strip.Color(r, g, b);
        strip.setPixelColor(pixel , c); // Draw new pixel
      }
      yEven = !yEven;
    }
    strip.setPin(panelPin[panel]);
    strip.show();
  }
    delay(10);  // you can change this number to change the scrolling speed.  Smaller number will be faster scrolling.

  // update sine values
  for (y=0; y<numRows; y++) {
    value = ysine[y][displayCols-1];
    for (x=displayCols-1; x>=0; x--) {
       ysine[y][x]=ysine[y][x-1];
    }
    ysine[y][0] = value;
  }

}

